Attribute VB_Name = "HEXIO"
Const BUFFER_SIZE = 1024
Const BUFFER_MASK = BUFFER_SIZE - 1
Const BUFFER_FILL = -1


Dim HEXData(BUFFER_SIZE) As Integer
Dim HEXAddrPtr As Long
Dim HEXFileCmpStr As String


Function HEXLoadMemBuf(HEXFileName As String, byteAddress) As Integer
    Dim FileLine As String
    Dim LineDataCount As Byte
    Dim LineAddrHigh As Long
    Dim LineAddr As Long
    Dim LineCode As Byte
    Dim FullAddr As Long
    
    HEXAddrPtr = (byteAddress \ BUFFER_SIZE) * BUFFER_SIZE
    HEXFileCmpStr = HEXFileName
    
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set HEXInFile = fs.OpenTextFile(HEXFileName, 1, False, 0)
    
    'Prime the buffer
    For i = 0 To BUFFER_SIZE - 1
        HEXData(i) = BUFFER_FILL
    Next i
    
    Do While HEXInFile.AtEndOfStream <> True
        'Read in a line
        FileLine = HEXInFile.ReadLine
        
        If Mid(FileLine, 1, 1) = ":" Then
            
            'Parse and store important line info
            LineDataCount = Val("&H" & Mid(FileLine, 2, 2))
            LineAddr = Val("&H1" & Mid(FileLine, 4, 4)) And 65535
            LineCode = Val("&H" & Mid(FileLine, 8, 2))
            
            FullAddr = (LineAddrHigh * 65536) + LineAddr
                                    
            'Decode the line code
            Select Case LineCode
                Case 0
                    For i = 0 To (LineDataCount - 1)
                        If (FullAddr >= HEXAddrPtr) And (FullAddr < (HEXAddrPtr + BUFFER_SIZE)) Then
                            HEXData(FullAddr And BUFFER_MASK) = Val("&H1" & Mid(FileLine, i * 2 + 10, 2)) And 255
                        End If
                        FullAddr = FullAddr + 1
                    Next i
                Case 4
                    LineAddrHigh = (Val("&H1" & Mid(FileLine, 10, 4)) And 65535)
                Case 1
                    Exit Do
            End Select
        End If
    Loop
    
    HEXInFile.Close
End Function


'Determine if the specified HEX file is valid
Function HEXValidata(HEXFileName As String) As Boolean
    Dim Checksum As Integer
    Dim InFileLine As String
    Dim DataCount As Integer
    Dim AddrCode As Integer
    Dim Address As Long
    Dim DataByte As Integer
        
    On Error GoTo ErrorHandler
    
        
    Set fs = CreateObject("Scripting.FileSystemObject")
    ChDir VB.App.Path
    Set InFile = fs.OpenTextFile(HEXFileName, 1, False, 0)
    
    'Check for an empty file
    If InFile.AtEndOfStream = True Then
        HEXValidata = False
        InFile.Close
        Exit Function
    End If
    
    'Validate the file before using it
    Do While InFile.AtEndOfStream <> True
        InFileLine = InFile.ReadLine
        
        AddrCode = 0
        If Mid(InFileLine, 1, 1) = "" Then
            DataByte = Asc(" ")
        Else
            DataByte = Asc(Mid(InFileLine, 1, 1))
        End If
        
        'check the line
        Select Case DataByte
            Case Asc(":")
                AddrCode = Val("&H" & Mid(InFileLine, 8, 2))
                DataCount = Val("&H" & Mid(InFileLine, 2, 2))
                Address = Val("&H1" & Mid(InFileLine, 4, 4)) And 65535
            
                Checksum = 0
                For i = 0 To DataCount + 4
                    Checksum = Checksum + Val("&H" & Mid(InFileLine, (2 * i) + 2, 2))
                Next i
            
                If (Checksum And 255) <> 0 Then
                    HEXValidata = False
                    InFile.Close
                    Exit Function
                End If
            Case Asc(" "), Asc(vbTab), Asc(vbCr), Asc(vbLf)
            Case Else
                HEXValidata = False
                InFile.Close
                Exit Function
        End Select
        
        If AddrCode = 1 Then
            Exit Do
        End If
        
        If InFile.AtEndOfStream = True Then
            HEXValidata = False
            InFile.Close
            Exit Function
        End If
    Loop
    
    HEXValidata = True
    
    InFile.Close
    Exit Function
    
ErrorHandler:
    Err.Clear
    HEXValidata = False
    InFile.Close
End Function


'Get the maximum address in the file
Function HEXUpper(HEXFileName As String) As Long

    Dim FileLine As String
    Dim LineDataCount As Byte
    Dim LineAddrHigh As Long
    Dim LineAddr As Long
    Dim LineCode As Byte
    Dim FullAddr As Long
    
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set HEXInFile = fs.OpenTextFile(HEXFileName, 1, False, 0)
    
    Do While HEXInFile.AtEndOfStream <> True
        'Read in a line
        FileLine = HEXInFile.ReadLine
        
        If Mid(FileLine, 1, 1) = ":" Then
            
            'Parse and store important line info
            LineDataCount = Val("&H" & Mid(FileLine, 2, 2))
            LineAddr = Val("&H1" & Mid(FileLine, 4, 4)) And 65535
            LineCode = Val("&H" & Mid(FileLine, 8, 2))
                        
            If LineCode = 4 Then LineAddrHigh = (Val("&H1" & Mid(FileLine, 10, 4)) And 65535)
            If LineCode = 1 Then Exit Do
            
            FullAddr = (LineAddrHigh * 65536) + LineAddr
            
            If LineCode = 0 Then
                If (FullAddr + LineDataCount - 1) > HEXUpper Then HEXUpper = (FullAddr + LineDataCount - 1)
            End If
        End If
    Loop
 
    HEXInFile.Close
 
End Function


'Get the minimum address in the file
Function HEXLower(HEXFileName As String) As Long
    Dim FileLine As String
    Dim LineDataCount As Byte
    Dim LineAddrHigh As Long
    Dim LineAddr As Long
    Dim LineCode As Byte
    Dim FullAddr As Long
    
    HEXLower = &H7FFFFFFF
    
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set HEXInFile = fs.OpenTextFile(HEXFileName, 1, False, 0)
    
    Do While HEXInFile.AtEndOfStream <> True
        'Read in a line
        FileLine = HEXInFile.ReadLine
        
        If Mid(FileLine, 1, 1) = ":" Then
            
            'Parse and store important line info
            LineDataCount = Val("&H" & Mid(FileLine, 2, 2))
            LineAddr = Val("&H1" & Mid(FileLine, 4, 4)) And 65535
            LineCode = Val("&H" & Mid(FileLine, 8, 2))
                        
            If LineCode = 4 Then LineAddrHigh = (Val("&H1" & Mid(FileLine, 10, 4)) And 65535)
                            
            If LineCode = 1 Then Exit Do
            
            FullAddr = (LineAddrHigh * 65536) + LineAddr
            
            If LineCode = 0 Then
                If FullAddr < HEXLower Then HEXLower = FullAddr
            End If
        End If
    Loop
    
    HEXInFile.Close
End Function


'Return the number of data bytes in the file
Function HEXLen(HEXFileName As String) As Long
    Dim FileLine As String
    Dim LineDataCount As Byte
    Dim LineCode As Byte
         
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set HEXInFile = fs.OpenTextFile(HEXFileName, 1, False, 0)
    
    Do While HEXInFile.AtEndOfStream <> True
    
        'Read in a line
        FileLine = HEXInFile.ReadLine
        
        If Mid(FileLine, 1, 1) = ":" Then
            
            'Parse and store important line info
            LineDataCount = Val("&H" & Mid(FileLine, 2, 2))
            LineCode = Val("&H" & Mid(FileLine, 8, 2))
                                                    
            If LineCode = 1 Then Exit Do
                        
            If LineCode = 0 Then
                HEXLen = HEXLen + LineDataCount
            End If
        End If
    Loop
    
    HEXInFile.Close
End Function


'Read a byte from the HEX file, if it does not exist then return 255
Function HEXRead(HEXFileName As String, byteAddr As Long) As Integer
    
    'Refresh buffer if different file names
    If StrComp(HEXFileName, HEXFileCmpStr) <> 0 Then
        HEXLoadMemBuf HEXFileName, byteAddr
    End If
    
    'Refresh buffer if address out of buffer's range
    If (byteAddr \ BUFFER_SIZE) <> (HEXAddrPtr \ BUFFER_SIZE) Then
        HEXLoadMemBuf HEXFileName, byteAddr
    End If
    
    HEXRead = HEXData(byteAddr And BUFFER_MASK)
End Function

















